from django.utils.translation import gettext_lazy as _
from rest_framework.exceptions import APIException, status


class AuthenticationFailed(APIException):
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = _('Incorrect authentication credentials.')
    default_code = 'authentication_failed'



from rest_framework.decorators import authentication_classes, permission_classes,\
    api_view
from rest_framework.permissions import AllowAny, BasePermission
from rest_framework.response import Response
import traceback
from .exceptions import AuthenticationFailed
from django.contrib.auth import authenticate
from django.conf import settings
from django.utils import timezone
from datetime import timedelta
import jwt
from rest_framework.authentication import BaseAuthentication,\
    get_authorization_header
from django.utils.translation import gettext as _
from rest_framework import HTTP_HEADER_ENCODING

from django.contrib.admin.sites import site as inner_site


def create_token(user, **payload):
    salt = settings.SECRET_KEY
    headers = {
        "typ": "jwt_",
        "alg": "HS256",
    }
    payload['user_id'] = user.id
    payload['username'] = user.username
    payload['exp'] = timezone.now() + timedelta(days=1)
    return jwt.encode(payload=payload, key=salt, headers=headers)

def parse_token(source):
    from django.contrib.auth.models import User
    token = jwt.decode(source, settings.SECRET_KEY, algorithms=["HS256"])
    user = User.objects.get(id=token['user_id'])
    return (user, token)

class IsAdminUser(BasePermission):
    """
    Allows access only to admin users.
    """
    def has_permission(self, request, view):
        from django.contrib.auth.models import User
        user = request.user
        return bool(user and
                    isinstance(user, User) and
                    user.is_staff and user.is_active)

class Authentication(BaseAuthentication):
    keyword = 'bearer'

    def authenticate(self, request):
        auth_header = get_authorization_header(request)
        auth_header = auth_header.decode(HTTP_HEADER_ENCODING)
        auth = auth_header.split()
        if not auth or auth[0].lower() != self.keyword:
            raise AuthenticationFailed()
        token = auth[1]
        try:
            user, token = parse_token(token)
            return (user, token)
        except jwt.exceptions.ExpiredSignatureError:
            raise AuthenticationFailed(detail='签名失效')
        except jwt.exceptions.DecodeError:
            raise AuthenticationFailed(detail='验证失败')
        except jwt.exceptions.InvalidTokenError:
            raise AuthenticationFailed(detail='非法token')

    def authenticate_header(self, _):
        return self.keyword

@api_view(['POST'])
@authentication_classes([])
@permission_classes([AllowAny])
def login(request):
    try:
        username = request.data['username']
        password = request.data['password']
        user = authenticate(username=username, password=password)
        if not user or not user.is_staff:
            raise Exception('验证失败')
        result = {
            'token': create_token(user)
        }
        return Response({'result': result})
    except Exception as e:
        traceback.print_exc()
        raise AuthenticationFailed(detail=str(e))

@api_view(['POST'])
@authentication_classes([])
@permission_classes([AllowAny])
def logout(request):
    return Response({})

@api_view(['GET'])
@authentication_classes([Authentication])
@permission_classes([IsAdminUser])
def user_info(request):
    def format_permission_id(model):
        meta = model._meta
        return f'{meta.app_label}-{meta.model_name}'
    from .sites import site
    user = request.user
    perms = site.perms(request)
    permissions = [
        {
            'permissionId': 'dashboard'
        }
    ]
    for app, app_info in perms.items():
        for model, actions in app_info['models']:
            if len(actions) == 0:
                continue
            permission = {
                'permissionId': format_permission_id(model),
                'actionList': actions,
            }
            permissions.append(permission)
    menus = [
        {
            'name': 'dashboard',
            'key': 'dashboard',
            'component': 'dashboard',
            'meta': {
                'icon': 'dashboard',
                'title': _('Site administration'),
            }
        }
    ]
    for app, app_info in perms.items():
        nav = {
            'name': app,
            'key': app,
            'component': 'menu',
            'meta': {
                'icon': 'table',
                'title': app_info['label']
            },
            'children': [],
        }
        for model, actions in app_info['models']:
            if len(actions) == 0:
                continue
            nav['children'].append({
                'name': model._meta.label.lower(),
                'key': model._meta.model_name.lower(),
                'component': 'django',
                'meta': {
                    'title': model._meta.verbose_name,
                    'permission': format_permission_id(model),
                }
            })
        menus.append(nav)
    result = {
        'id': user.id,
        'username': user.username,
        'role': {
            'permissions': permissions
        },
        'sitename': inner_site.site_header,
        'menus': menus
    }
    return Response({'result': result})



from django.contrib.admin.options import get_content_type_for_model

def log_addition(request, object, message):
    """
    Log that an object has been successfully added.

    The default implementation creates an admin LogEntry object.
    """
    from django.contrib.admin.models import LogEntry, ADDITION
    return LogEntry.objects.log_action(
        user_id=request.user.pk,
        content_type_id=get_content_type_for_model(object).pk,
        object_id=object.pk,
        object_repr=str(object),
        action_flag=ADDITION,
        change_message=message,
    )

def log_change(request, object, message):
    """
    Log that an object has been successfully changed.

    The default implementation creates an admin LogEntry object.
    """
    from django.contrib.admin.models import LogEntry, CHANGE
    return LogEntry.objects.log_action(
        user_id=request.user.pk,
        content_type_id=get_content_type_for_model(object).pk,
        object_id=object.pk,
        object_repr=str(object),
        action_flag=CHANGE,
        change_message=message,
    )

def log_deletion(request, object, object_repr):
    """
    Log that an object will be deleted. Note that this method must be
    called before the deletion.

    The default implementation creates an admin LogEntry object.
    """
    from django.contrib.admin.models import LogEntry, DELETION
    return LogEntry.objects.log_action(
        user_id=request.user.pk,
        content_type_id=get_content_type_for_model(object).pk,
        object_id=object.pk,
        object_repr=object_repr,
        action_flag=DELETION,
    )


from .sites import site
from django.contrib import admin


def patch_register(*models, inner_site=None):
    """
    Register the given model(s) classes and wrapped ModelAdmin class with
    admin site:

    @register(Author)
    class AuthorAdmin(admin.ModelAdmin):
        pass

    The `site` kwarg is an admin site to use instead of the default admin site.
    """
    from django.contrib.admin import ModelAdmin
    from django.contrib.admin.sites import site as default_site, AdminSite

    def _model_admin_wrapper(admin_class):
        if not models:
            raise ValueError('At least one model must be passed to register.')

        admin_site = inner_site or default_site

        if not isinstance(admin_site, AdminSite):
            raise ValueError('site must subclass AdminSite')

        if not issubclass(admin_class, ModelAdmin):
            raise ValueError('Wrapped class must subclass ModelAdmin.')

        admin_site.register(models, admin_class=admin_class)
        
        site.register(models, admin_class)

        return admin_class
    return _model_admin_wrapper

admin.register = patch_register




from .viewsets import GeneralViewSet, GeneralListSerializer, GeneralSerializer
from . import auth
from .core import parse_model, has_model_attr, ModelInfo, set_password
from .pagination import PagePagination
from .fields import PasswordField
from rest_framework.routers import SimpleRouter
from rest_framework import serializers
from django.urls.conf import path
from rest_framework.decorators import MethodMapper
from dataclasses import field
from django.db.models.fields.related import ManyToManyField, ForeignKey

class DefaultSite():
    router: SimpleRouter
    actions = {}
    def __init__(self, name='admin-api'):
        self._registry = {} # ModelAdmin内部get_fieldset会使用
        self.name = name
        self.router = SimpleRouter(trailing_slash=True)
        
    @property
    def urls(self):
        urlpatterns =  self.router.urls
        urlpatterns += (
            path('auth/login/', auth.login),
            path('auth/logout/', auth.logout),
            path('auth/userinfo/', auth.user_info),
        )
        return urlpatterns, 'admin-api', self.name
    
    def _patch_fields(self, admin, args):
        # patch password
        for name, _ in admin.passwords:
            if admin.fields and name not in admin.fields:
                continue
            field = admin.model._meta.get_field(name)
            args[name] = serializers.CharField(write_only=True, label=field.verbose_name, style={'input_type': 'password'})

    def _gen_admin_class(self, model, admin_class):
        from django.contrib.auth.models import User
        admin_class = type('',(admin_class,), {})
        if model == User:
            # 给官方的UserAdmin打一下补丁
            # 官方的新增写法不标准（password1, password2），按drf协议修正一下
            admin_class.add_fieldsets = ((None, {'fields': ('username', 'password')}),)
            # 官方UserAdmin为了显示新增页面覆盖了get_fieldsets， 这里需要修正一下
            admin_class.get_fieldsets = eval('lambda self, request: self.fieldsets')
            admin_class.passwords = (('password',set_password),)
        else:
            # 自实现的PasswordField处理
            passwords = []
            for field in model._meta.get_fields():
                if type(field) == PasswordField:
                    passwords.append((field.name, field.method))
            admin_class.passwords = passwords
        
        # 为admin_class注入2个属性（如果没有的话）
        for name in ('add_fieldsets', 'passwords'):
            if hasattr(admin_class, name):
                continue
            setattr(admin_class, name, ())
        return admin_class(model, self)
    
    def _build_pagination_class(self, admin):
        pagination_class = type('', (PagePagination,), {'page_size': admin.list_per_page})
        return pagination_class
    
    def _build_list_serialize_class(self, admin):
        args = {
            'pk': serializers.CharField(),
        }
        if not isinstance(admin.list_display, str):
            for attr_name in admin.list_display:
                if attr_name.startswith('__'):
                    continue
                if has_model_attr(admin, attr_name) or hasattr(admin, attr_name):
                    args[attr_name] = serializers.SerializerMethodField()
                    args[f'get_{attr_name}'] = eval(f'lambda self, obj: self._get_admin_attr_value("{attr_name}", obj)')
        serializer_class = type('', (GeneralListSerializer, ), args)
        serializer_class._admin = admin
        serializer_class.Meta = type('', (), {})
        serializer_class.Meta.model = admin.model
        if admin.list_display == '__all__':
            serializer_class.Meta.fields = '__all__'
        else:
            serializer_class.Meta.fields = ['pk']
            serializer_class.Meta.fields.extend(admin.list_display)
        return serializer_class
    
    def _build_autocomplete_serialize_class(self, admin):
        args = {
            'key': serializers.CharField(source='pk'),
            'label': serializers.CharField(source='__str__')
        }
        serializer_class = type('', (GeneralListSerializer, ), args)
        serializer_class.Meta = type('', (), {})
        serializer_class.Meta.model = admin.model
        serializer_class.Meta.fields = ('key', 'label')
        return serializer_class
    
    def _build_view_serialize_class(self, admin):
        args = {
        }
        self._patch_fields(admin, args)
        for field in admin.opts.get_fields():
            t = type(field)
            if t in (ManyToManyField,):
                args[field.name] = serializers.StringRelatedField(many=True)
            elif t in (ForeignKey,):
                args[field.name] = serializers.StringRelatedField(many=False)
        serializer_class: serializers.ModelSerializer = type('', (GeneralSerializer, ), args)
        serializer_class._admin = admin
        serializer_class.Meta = type('', (), {})
        serializer_class.Meta.model = admin.model
        serializer_class.Meta.fields = admin.fields if admin.fields else '__all__'
        return serializer_class
        
    
    def _build_serialize_class(self, admin):
        args = {
        }
        self._patch_fields(admin, args)
        serializer_class: serializers.ModelSerializer = type('', (GeneralSerializer, ), args)
        serializer_class._admin = admin
        serializer_class.Meta = type('', (), {})
        serializer_class.Meta.model = admin.model
        serializer_class.Meta.fields = admin.fields if admin.fields else '__all__'
        return serializer_class
    
    
    def _build_actions(self, viewset_class):
        def insert_bulk_action(name):
            def innerfunc(self: GeneralViewSet, request):
                return self._call_admin_action(name, request)
            innerfunc.__name__ = f'action_{name}'
            innerfunc.mapping = MethodMapper(innerfunc, methods=('post',))
            innerfunc.detail = False
            innerfunc.url_path = innerfunc.__name__
            innerfunc.url_name = innerfunc.__name__.replace('_', '-')
            innerfunc.kwargs = {}
            setattr(viewset_class, innerfunc.__name__, innerfunc)
        actions = viewset_class._admin.actions
        if actions is None:
            return
        insert_bulk_action('delete')
        for name in actions:
            insert_bulk_action(name)
    
    def perms(self, request):
        def perm_list(admin):
            actions = []
            for action, perm in admin.get_model_perms(request).items():
                if perm:
                    actions.append(action)
            return actions 
        rs = {}
        for model, admin in self._registry.items():
            info: ModelInfo = parse_model(model)
            if not request.user.has_module_perms(info.app_name):
                continue
            actions = perm_list(admin)
            if len(actions) == 0:
                continue
            if info.app_name not in rs:
                rs[info.app_name] = {
                    'label': info.app_label,
                    'models': []
                }
            rs[info.app_name]['models'].append((model, actions))
        return rs

    def register(self, models, admin_class):
        if not isinstance(models, list) and not isinstance(models, tuple):
            models = [models]
                    
        for model in models:
            admin = self._gen_admin_class(model, admin_class)
            info: ModelInfo = parse_model(model)
            viewset_classname = f'{info.model_name}ViewSet'
            viewset_class: GeneralViewSet = type(viewset_classname, (GeneralViewSet,), {})
            viewset_class.queryset = model.objects.all()
            viewset_class.search_fields = admin.search_fields
            viewset_class.filterset_fields = admin.list_filter
            viewset_class._admin = admin
            viewset_class.list_serializer_class = self._build_list_serialize_class(admin)
            viewset_class.autocomplete_serializer_class = self._build_autocomplete_serialize_class(admin)
            viewset_class.serializer_class = self._build_serialize_class(admin)
            viewset_class.view_serializer_class = self._build_view_serialize_class(admin)
            viewset_class.pagination_class = self._build_pagination_class(admin)
            self._build_actions(viewset_class)
            self._registry[model] = admin
            self.router.register(info.prefix, viewset_class)

site = DefaultSite()


from django.db.models import CharField
from .core import set_password

class PasswordField(CharField):
    def __init__(self, *args, **kwargs):
        self.method = kwargs.pop('method', set_password)
        super().__init__(*args, **kwargs)


from .auth import Authentication
from .pagination import PagePagination
from .core import get_passwords, ModelInfo
from .core import parse_model, get_short_description, get_field_title
from .core import has_model_attr
from . import log
from rest_framework import viewsets, serializers
from rest_framework.metadata import SimpleMetadata
from rest_framework.permissions import BasePermission
from rest_framework import filters
from rest_framework.request import clone_request
from rest_framework import exceptions
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.relations import PrimaryKeyRelatedField, ManyRelatedField
from rest_framework.fields import DecimalField
from django_filters.rest_framework.backends import DjangoFilterBackend
from django.contrib.admin.options import ModelAdmin
from django.contrib import messages
from django.db import models
from django.http import Http404
from django.core.exceptions import PermissionDenied
from django.db.models import fields

class GeneralMetadata(SimpleMetadata):
    _view = None
    def __init__(self):
        pass
    def _get_filters(self, view):
        admin = view._admin
        list_filter = []
        for key in admin.list_filter:
            field = admin.opts.get_field(key)
            title = field.verbose_name
            choices = field.choices
            if not choices:
                choices = []
                t = type(field)
                if t == fields.related.ManyToManyField:
                    for k, v in field.get_choices():
                        if k:
                            choices.append((k, v))
                elif t == fields.BooleanField:
                    choices = (
                        (1, '是'),
                        (0, '否'),
                    )
            list_filter.append({
                'key': key,
                'label': title,
                'choices': choices,
            })
        search_fields = []
        for key in admin.search_fields:
            field = admin.opts.get_field(key)
            title = field.verbose_name
            search_fields.append({
                'key': key,
                'label': title,
            })
        rs = {
            'listFilter': list_filter,
            'searchFields': search_fields,
        }
        return rs

    def _patch_input_type(self, field, data):
        def is_password_type(admin, fieldname):
            for name, _ in admin.passwords:
                if name == fieldname:
                    return True
            return False
        try:
            source = self._view._admin.opts.get_field(field.source)
            if source.default != fields.NOT_PROVIDED:
                default = source.default
                if callable(default):
                    default = default()
                data['default'] = default
        except:
            pass
        if is_password_type(self._view._admin, field.source):
            data['input_type'] = 'password'
        elif 'base_template' in field.style:
            if field.style['base_template'] == 'textarea.html':
                data['input_type'] = 'textarea'

    def determine_metadata(self, request, view):
        self._view = view
        data = SimpleMetadata.determine_metadata(self, request, view)
        data['fieldsets'] = view._admin.get_fieldsets(request)
        data['addFieldsets'] = view._admin.add_fieldsets
        data['filters'] = self._get_filters(view)
        data['pageSize'] = self._view._admin.list_per_page
        return data

    def determine_actions(self, request, view):
        """
        复制底层代码，修改权限验证逻辑
        """ 
        def check_permission(view):
            for action in ('view', 'change', 'add'):
                attrname = f'has_{action}_permission'
                if getattr(view._admin, attrname)(view.request):
                    return True
        actions = {}
        for method in {'PUT', 'POST'} & set(view.allowed_methods):
            view.request = clone_request(request, method)
            try:
                # Test global permissions
                # if hasattr(view, 'check_permissions'):
                #     view.check_permissions(view.request)
                if not check_permission(view):
                    continue
                
                # Test object permissions
                if method == 'PUT' and hasattr(view, 'get_object'):
                    view.get_object()
            except (exceptions.APIException, PermissionDenied, Http404):
                pass
            else:
                # If user has appropriate permissions for the view, include
                # appropriate metadata about the fields that should be supplied.
                serializer = view.get_serializer()
                actions[method] = self.get_serializer_info(serializer)
            finally:
                view.request = request

        return actions

    def get_field_info(self, field):
        data = SimpleMetadata.get_field_info(self, field)
        data['write_only'] = field.write_only
        self._patch_input_type(field, data)
        if isinstance(field, PrimaryKeyRelatedField):
            from .sites import site
            model = field.queryset.model
            info: ModelInfo = parse_model(model)
            relation = {
                'app': info.app_name,
                'model': info.model_name,
                'lazy': False, # 如果关联模型设置了search_fields则使用autocomplete懒加载，否则全部显示，阅读前端代码ForeignSelect.vue实现
            }
            admin = site._registry.get(model)
            if admin and admin.search_fields:
                relation['lazy'] = True
            data['relation'] = relation
            data['field_type'] = 'ManyToOne'
        elif isinstance(field, ManyRelatedField):
            data['choices'] = field.get_choices()
            data['field_type'] = 'ManyToMany'
        elif isinstance(field, DecimalField):
            data['decimal_places'] = field.decimal_places
        return data

class GeneralPermission(BasePermission):
    perms_map = {
        'GET': 'view',
        'OPTIONS': '',
        'HEAD': '',
        'POST': 'add',
        'PUT': 'change',
        'PATCH': 'change',
        'DELETE': 'delete',
    }
    def has_object_permission(self, request, view, obj):
        return self.has_permission(request, view)
    
    def has_permission(self, request, view):
        action = self.perms_map.get(request.method)
        if not action:
            return True
        attrname = f'has_{action}_permission'
        return getattr(view._admin, attrname)(request)
        

class GeneralSerializer(serializers.ModelSerializer):
    _admin: ModelAdmin = None
    @property
    def request(self):
        return self.context['request']
    
    def _del_file(self, instance):
        def filter_del_keys():
            prefix = '__del__'
            rs = []
            for key in self.initial_data.keys():
                if key.find(prefix) != 0:
                    continue
                rs.append(key[len(prefix):])
            return rs
        for key in filter_del_keys():
            # 删除文件
            field = getattr(instance, key)
            field.delete(save=False)
    def _set_password(self, validated_data):
        passwords = get_passwords(self._admin)
        for k in validated_data.keys():
            if k not in passwords:
                continue
            func = passwords[k]
            if not callable(func):
                func = getattr(self._admin, func)
            validated_data[k] = func(validated_data, validated_data[k])
    def update(self, instance, validated_data):
        def modify_fields():
            rs = []
            for k, v in validated_data.items():
                oldv = getattr(instance, k)
                if oldv != v:
                    rs.append(k)
            return rs
        self._del_file(instance)
        self._set_password(validated_data)
        fields = modify_fields()
        obj = serializers.ModelSerializer.update(self, instance, validated_data)
        message = []
        if len(fields) > 0:
            item = {
                'changed': {
                'fields': fields,
                }
            }
            message.append(item)
        log.log_change(self.request, obj, message)
        return obj
    
    def create(self, validated_data):
        self._set_password(validated_data)
        obj = serializers.ModelSerializer.create(self, validated_data)
        message = [{'added': {}}]
        log.log_addition(self.request, obj, message)
        return obj
    

class GeneralListSerializer(serializers.ModelSerializer):
    _admin: ModelAdmin = None
    
    def _get_admin_attr_value(self, attr_name, obj):
        if has_model_attr(self._admin, attr_name):
            return getattr(obj, attr_name)()
        return getattr(self._admin, attr_name)(obj)

        
class GeneralViewSet(viewsets.ModelViewSet):
    _admin: ModelAdmin = None
    list_serializer_class = None
    view_serializer_class = None
    autocomplete_serializer_class = None
    pagination_class = PagePagination
    filter_backends = [DjangoFilterBackend, filters.SearchFilter]

    metadata_class = GeneralMetadata
    authentication_classes = [Authentication,]
    permission_classes = [GeneralPermission,]
    
 
    def _inner_delete(self, request, queryset):
        for obj in queryset:
            log.log_deletion(request, obj, str(obj))
        count, _ = queryset.delete()
        message = f'{count}条记录被删除'
        self._admin.message_user(request, message, messages.SUCCESS)
    
    def _call_admin_action(self, name, request):
        if 'all' in request.data:
            queryset = self.get_queryset()
        else:
            ids = request.data.get('ids', [])
            queryset = self.get_queryset().filter(pk__in=ids)
        if name == 'delete':
            func = self._inner_delete
        else:
            func = getattr(self._admin, name)
        func(request, queryset)
        msgs = []
        for msg in messages.get_messages(request):
            msgs.append({'level': msg.level, 'message': msg.message})
        
        return Response({'status': 0, 'message': msgs})
    
    @property
    def _model(self) -> models.Model:
        return self.serializer_class.Meta.model
    
    @property
    def _slots(self):
        if not hasattr(self._admin, 'slots'):
            return {}
        rs = {}
        for k, v in self._admin.slots.items():
            if isinstance(v, str):
                path = v
                params = {}
            else:
                path, params = v
            rs[k] = (path, params)
        return rs
    
    def _get_actions(self, request):
        actions = []
        origin_actions = self._admin.get_actions(request)
        if origin_actions != None:
            if self._admin.has_delete_permission(request):
                actions.append({'name': '删除', 'action': 'action_delete'})
            for item in origin_actions:
                if not hasattr(self._admin, item):
                    continue
                name = get_short_description(self._admin, item)
                actions.append({'name': name, 'action': f'action_{item}'})
        return {
            'items': actions,
            'onTop': self._admin.actions_on_top,
            'onButton': self._admin.actions_on_bottom,
            'showSelection': self._admin.actions_selection_counter
        }
    
    @property
    def _columns(self):
        rs = []
        for field in self._admin.list_display:
            if field == 'pk':
                continue
            column = {
                'title': get_field_title(self._admin, field),
                'dataIndex': field,
            }
            if field in self._slots:
                column['scopedSlots'] = {'customRender': field}
            rs.append(column)
        return rs
    
    def list(self, request, *args, **kwargs):
        self.serializer_class = self.list_serializer_class
        data = viewsets.ModelViewSet.list(self, request, *args, **kwargs)
        data.data['columns'] = self._columns
        data.data['slots'] = self._slots
        data.data['actions'] = self._get_actions(request)
        return data

    @action(detail=False)
    def autocomplete(self, request, *args, **kwargs):
        self.serializer_class = self.autocomplete_serializer_class
        return viewsets.ModelViewSet.list(self, request, *args, **kwargs)
    
    @action(detail=True)
    def view(self, request, *args, **kwargs):
        self.serializer_class = self.view_serializer_class
        return viewsets.ModelViewSet.retrieve(self, request, *args, **kwargs)
    
    @action(detail=True)
    def str(self, request, *args, **kwargs):
        instance = self.get_object()
        return Response({'str': instance.__str__()})
    
    def perform_destroy(self, instance):
        log.log_deletion(self.request, instance, str(instance))
        viewsets.ModelViewSet.perform_destroy(self, instance)



from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response
from django.core.paginator import EmptyPage


class PagePagination(PageNumberPagination):
    page = None
    request = None
    page_size = 100
    page_size_query_param = 'pageSize'
    page_query_param = 'pageNo'

    def get_paginated_response(self, data):
        result = {
            'data': data,
            'pageSize': self.page_size,
            'pageNo': self.page.number,
            'totalPage': self.page.paginator.num_pages,
            'totalCount': self.page.paginator.count,
        }
        response = {
            'message': '',
            'status': 200,
            'result': result,
        }
        return Response(response)

    def paginate_queryset(self, queryset, request, view=None):
        if request.query_params.get(self.page_size_query_param) == '-1':
            self.page_size = 9999999 # 设置一个极限值，逻辑比较简单
        self.page_size = self.get_page_size(request)
        page_number = request.query_params.get(self.page_query_param, 1)
        paginator = self.django_paginator_class(queryset, self.page_size)
        try:
            self.page = paginator.page(page_number)
        except EmptyPage:
            self.page = paginator.page(paginator.num_pages)
        self.request = request
        return list(self.page)



from django.db import models
from django.db.models.options import Options
from django.utils.text import capfirst
from django.core.exceptions import FieldDoesNotExist
from django.contrib.auth.hashers import make_password


class ModelInfo:
    app_name = None
    app_label = None
    model_name = None
    model_label = None
    prefix = None

def parse_model(model: models.Model) -> ModelInfo:
    meta: Options = model._meta
    rs = ModelInfo
    rs.app_name = meta.app_config.label.lower()
    rs.app_label = meta.app_config.verbose_name
    rs.model_name = meta.model_name.lower()
    rs.model_label = meta.verbose_name
    rs.prefix = f'resource/{rs.app_name}/{rs.model_name}'
    return rs


def get_short_description(obj, attr_name):
    attr = getattr(obj, attr_name)
    if hasattr(attr, 'short_description'):
        return getattr(attr, 'short_description')
    else:
        return capfirst(attr.replace('_', ' '))

def is_model_field(admin, name):
    try:
        admin.opts.get_field(name)
        return True
    except FieldDoesNotExist:
        return False

def has_model_attr(admin, name):
    if is_model_field(admin, name):
        return False
    return hasattr(admin.model, name)

def get_model_field_or_attr_name(admin, name):
    try:
        return admin.opts.get_field(name).verbose_name
    except FieldDoesNotExist:
        if hasattr(admin.model, name):
            return get_short_description(admin.model, name)

def get_field_title(admin, field):
    if field == '__str__':
        return admin.model._meta.verbose_name
    title = get_model_field_or_attr_name(admin, field)
    if title:
        return title
    return get_short_description(admin, field)

def get_passwords(admin):
    rs = {}
    for name, funcname in admin.passwords:
        rs[name] = funcname
    return rs

def set_password(_, rawPassword):
    return make_password(rawPassword)



from setuptools import setup

def read(f):
    return open(f, 'r', encoding='utf-8').read()

setup(
    name='django-admin-api',
    version='0.0.7',
    packages=['admin_api'],
    zip_safe=False,
    include_package_data=True,
    license='BSD License',  # example license
    description='export Django Admin API by restful style',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    url='https://github.com/django-extend/django-admin-api.git',
    author='Nieo Ding',
    author_email='8285770@qq.com',
    install_requires = [
        'djangorestframework',
        'django-filter',
        'pyjwt',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)